#include "Misc/Constants.hpp"
#include "Signal.hpp"
#include "Register.hpp"

// ------------------------------------------------------------------------------------------------
#include <limits>
#include <cfloat>

// ------------------------------------------------------------------------------------------------
namespace SqMod {

// ------------------------------------------------------------------------------------------------
bool Register_Constants(HSQUIRRELVM vm)
{
    Sqrat::ConstTable(vm).Enum(_SC("SQMOD"), Sqrat::Enumeration(vm)
        .Const(_SC("VERSION"),      SQMOD_VERSION)
        .Const(_SC("SUCCESS"),      SQMOD_SUCCESS)
        .Const(_SC("FAILURE"),      SQMOD_FAILURE)
        .Const(_SC("UNKNOWN"),      SQMOD_UNKNOWN)
        .Const(_SC("ARCH"),         SQMOD_ARCHITECTURE)
        .Const(_SC("PLATFORM"),     SQMOD_PLATFORM)
        .Const(_SC("MIN_CHAR"),     _NLMIN(SQChar))
        .Const(_SC("MAX_CHAR"),     _NLMAX(SQChar))
        .Const(_SC("MIN_SHORT"),    _NLMIN(Int16))
        .Const(_SC("MAX_SHORT"),    _NLMAX(Int16))
        .Const(_SC("MIN_USHORT"),   _NLMIN(Uint16))
        .Const(_SC("MAX_USHORT"),   _NLMAX(Uint16))
        .Const(_SC("MIN_INT"),      _NLMIN(SQInteger))
        .Const(_SC("MAX_INT"),      _NLMAX(SQInteger))
        .Const(_SC("MIN_INT32"),    _NLMIN(SQInt32))
        .Const(_SC("MAX_INT32"),    _NLMAX(SQInt32))
        .Const(_SC("MIN_FLOAT"),    _NLMIN(SQFloat))
        .Const(_SC("MAX_FLOAT"),    _NLMAX(SQFloat))
    );

    Sqrat::ConstTable(vm).Enum(_SC("EARCHITECTURE"), Sqrat::Enumeration(vm)
        .Const(_SC("UNKNOWN"),      SQMOD_ARCH_ID_UNKNOWN)
        .Const(_SC("X32_BIT"),      SQMOD_ARCH_ID_32_BIT)
        .Const(_SC("X64_BIT"),      SQMOD_ARCH_ID_64_BIT)
    );

    Sqrat::ConstTable(vm).Enum(_SC("EPLATFORM"), Sqrat::Enumeration(vm)
        .Const(_SC("UNKNOWN"),      SQMOD_PLAT_ID_UNKNOWN)
        .Const(_SC("WINDOWS"),      SQMOD_PLAT_ID_WINDOWS)
        .Const(_SC("LINUX"),        SQMOD_PLAT_ID_LINUX)
        .Const(_SC("MACOS"),        SQMOD_PLAT_ID_MACOS)
        .Const(_SC("UNIX"),         SQMOD_PLAT_ID_UNIX)
    );

    Sqrat::ConstTable(vm).Enum(_SC("EVENTID"), Sqrat::Enumeration(vm)
        .Const(_SC("UNKNOWN"),                  EVT_UNKNOWN)
        .Const(_SC("BLIPCREATED"),              EVT_BLIPCREATED)
        .Const(_SC("CHECKPOINTCREATED"),        EVT_CHECKPOINTCREATED)
        .Const(_SC("KEYBINDCREATED"),           EVT_KEYBINDCREATED)
        .Const(_SC("OBJECTCREATED"),            EVT_OBJECTCREATED)
        .Const(_SC("PICKUPCREATED"),            EVT_PICKUPCREATED)
        .Const(_SC("PLAYERCREATED"),            EVT_PLAYERCREATED)
        .Const(_SC("SPHERECREATED"),            EVT_SPHERECREATED)
        .Const(_SC("SPRITECREATED"),            EVT_SPRITECREATED)
        .Const(_SC("TEXTDRAWCREATED"),          EVT_TEXTDRAWCREATED)
        .Const(_SC("VEHICLECREATED"),           EVT_VEHICLECREATED)
        .Const(_SC("BLIPDESTROYED"),            EVT_BLIPDESTROYED)
        .Const(_SC("CHECKPOINTDESTROYED"),      EVT_CHECKPOINTDESTROYED)
        .Const(_SC("KEYBINDDESTROYED"),         EVT_KEYBINDDESTROYED)
        .Const(_SC("OBJECTDESTROYED"),          EVT_OBJECTDESTROYED)
        .Const(_SC("PICKUPDESTROYED"),          EVT_PICKUPDESTROYED)
        .Const(_SC("PLAYERDESTROYED"),          EVT_PLAYERDESTROYED)
        .Const(_SC("SPHEREDESTROYED"),          EVT_SPHEREDESTROYED)
        .Const(_SC("SPRITEDESTROYED"),          EVT_SPRITEDESTROYED)
        .Const(_SC("TEXTDRAWDESTROYED"),        EVT_TEXTDRAWDESTROYED)
        .Const(_SC("VEHICLEDESTROYED"),         EVT_VEHICLEDESTROYED)
        .Const(_SC("BLIPCUSTOM"),               EVT_BLIPCUSTOM)
        .Const(_SC("CHECKPOINTCUSTOM"),         EVT_CHECKPOINTCUSTOM)
        .Const(_SC("KEYBINDCUSTOM"),            EVT_KEYBINDCUSTOM)
        .Const(_SC("OBJECTCUSTOM"),             EVT_OBJECTCUSTOM)
        .Const(_SC("PICKUPCUSTOM"),             EVT_PICKUPCUSTOM)
        .Const(_SC("PLAYERCUSTOM"),             EVT_PLAYERCUSTOM)
        .Const(_SC("SPHERECUSTOM"),             EVT_SPHERECUSTOM)
        .Const(_SC("SPRITECUSTOM"),             EVT_SPRITECUSTOM)
        .Const(_SC("TEXTDRAWCUSTOM"),           EVT_TEXTDRAWCUSTOM)
        .Const(_SC("VEHICLECUSTOM"),            EVT_VEHICLECUSTOM)
        .Const(_SC("PLAYERAWAY"),               EVT_PLAYERAWAY)
        .Const(_SC("PLAYERGAMEKEYS"),           EVT_PLAYERGAMEKEYS)
        .Const(_SC("PLAYERRENAME"),             EVT_PLAYERRENAME)
        .Const(_SC("PLAYERREQUESTCLASS"),       EVT_PLAYERREQUESTCLASS)
        .Const(_SC("PLAYERREQUESTSPAWN"),       EVT_PLAYERREQUESTSPAWN)
        .Const(_SC("PLAYERSPAWN"),              EVT_PLAYERSPAWN)
        .Const(_SC("PLAYERSTARTTYPING"),        EVT_PLAYERSTARTTYPING)
        .Const(_SC("PLAYERSTOPTYPING"),         EVT_PLAYERSTOPTYPING)
        .Const(_SC("PLAYERCHAT"),               EVT_PLAYERCHAT)
        .Const(_SC("PLAYERCOMMAND"),            EVT_PLAYERCOMMAND)
        .Const(_SC("PLAYERMESSAGE"),            EVT_PLAYERMESSAGE)
        .Const(_SC("PLAYERHEALTH"),             EVT_PLAYERHEALTH)
        .Const(_SC("PLAYERARMOUR"),             EVT_PLAYERARMOUR)
        .Const(_SC("PLAYERWEAPON"),             EVT_PLAYERWEAPON)
        .Const(_SC("PLAYERMOVE"),               EVT_PLAYERMOVE)
        .Const(_SC("PLAYERWASTED"),             EVT_PLAYERWASTED)
        .Const(_SC("PLAYERKILLED"),             EVT_PLAYERKILLED)
        .Const(_SC("PLAYERTEAMKILL"),           EVT_PLAYERTEAMKILL)
        .Const(_SC("PLAYERSPECTATE"),           EVT_PLAYERSPECTATE)
        .Const(_SC("PLAYERCRASHREPORT"),        EVT_PLAYERCRASHREPORT)
        .Const(_SC("PLAYERBURNING"),            EVT_PLAYERBURNING)
        .Const(_SC("PLAYERCROUCHING"),          EVT_PLAYERCROUCHING)
        .Const(_SC("PLAYERSTATE"),              EVT_PLAYERSTATE)
        .Const(_SC("PLAYERACTION"),             EVT_PLAYERACTION)
        .Const(_SC("STATENONE"),                EVT_STATENONE)
        .Const(_SC("STATENORMAL"),              EVT_STATENORMAL)
        .Const(_SC("STATESHOOTING"),            EVT_STATESHOOTING)
        .Const(_SC("STATEDRIVER"),              EVT_STATEDRIVER)
        .Const(_SC("STATEPASSENGER"),           EVT_STATEPASSENGER)
        .Const(_SC("STATEENTERDRIVER"),         EVT_STATEENTERDRIVER)
        .Const(_SC("STATEENTERPASSENGER"),      EVT_STATEENTERPASSENGER)
        .Const(_SC("STATEEXITVEHICLE"),         EVT_STATEEXITVEHICLE)
        .Const(_SC("STATEUNSPAWNED"),           EVT_STATEUNSPAWNED)
        .Const(_SC("ACTIONNONE"),               EVT_ACTIONNONE)
        .Const(_SC("ACTIONNORMAL"),             EVT_ACTIONNORMAL)
        .Const(_SC("ACTIONAIMING"),             EVT_ACTIONAIMING)
        .Const(_SC("ACTIONSHOOTING"),           EVT_ACTIONSHOOTING)
        .Const(_SC("ACTIONJUMPING"),            EVT_ACTIONJUMPING)
        .Const(_SC("ACTIONLIEDOWN"),            EVT_ACTIONLIEDOWN)
        .Const(_SC("ACTIONGETTINGUP"),          EVT_ACTIONGETTINGUP)
        .Const(_SC("ACTIONJUMPVEHICLE"),        EVT_ACTIONJUMPVEHICLE)
        .Const(_SC("ACTIONDRIVING"),            EVT_ACTIONDRIVING)
        .Const(_SC("ACTIONDYING"),              EVT_ACTIONDYING)
        .Const(_SC("ACTIONWASTED"),             EVT_ACTIONWASTED)
        .Const(_SC("ACTIONEMBARKING"),          EVT_ACTIONEMBARKING)
        .Const(_SC("ACTIONDISEMBARKING"),       EVT_ACTIONDISEMBARKING)
        .Const(_SC("VEHICLERESPAWN"),           EVT_VEHICLERESPAWN)
        .Const(_SC("VEHICLEEXPLODE"),           EVT_VEHICLEEXPLODE)
        .Const(_SC("VEHICLEHEALTH"),            EVT_VEHICLEHEALTH)
        .Const(_SC("VEHICLEMOVE"),              EVT_VEHICLEMOVE)
        .Const(_SC("PICKUPRESPAWN"),            EVT_PICKUPRESPAWN)
        .Const(_SC("KEYBINDKEYPRESS"),          EVT_KEYBINDKEYPRESS)
        .Const(_SC("KEYBINDKEYRELEASE"),        EVT_KEYBINDKEYRELEASE)
        .Const(_SC("VEHICLEEMBARKING"),         EVT_VEHICLEEMBARKING)
        .Const(_SC("VEHICLEEMBARKED"),          EVT_VEHICLEEMBARKED)
        .Const(_SC("VEHICLEDISEMBARK"),         EVT_VEHICLEDISEMBARK)
        .Const(_SC("PICKUPCLAIMED"),            EVT_PICKUPCLAIMED)
        .Const(_SC("PICKUPCOLLECTED"),          EVT_PICKUPCOLLECTED)
        .Const(_SC("OBJECTSHOT"),               EVT_OBJECTSHOT)
        .Const(_SC("OBJECTBUMP"),               EVT_OBJECTBUMP)
        .Const(_SC("CHECKPOINTENTERED"),        EVT_CHECKPOINTENTERED)
        .Const(_SC("CHECKPOINTEXITED"),         EVT_CHECKPOINTEXITED)
        .Const(_SC("SPHEREENTERED"),            EVT_SPHEREENTERED)
        .Const(_SC("SPHEREEXITED"),             EVT_SPHEREEXITED)
        .Const(_SC("SERVERFRAME"),              EVT_SERVERFRAME)
        .Const(_SC("SERVERSTARTUP"),            EVT_SERVERSTARTUP)
        .Const(_SC("SERVERSHUTDOWN"),           EVT_SERVERSHUTDOWN)
        .Const(_SC("INTERNALCOMMAND"),          EVT_INTERNALCOMMAND)
        .Const(_SC("LOGINATTEMPT"),             EVT_LOGINATTEMPT)
        .Const(_SC("CUSTOMEVENT"),              EVT_CUSTOMEVENT)
        .Const(_SC("WORLDOPTION"),              EVT_WORLDOPTION)
        .Const(_SC("WORLDTOGGLE"),              EVT_WORLDTOGGLE)
        .Const(_SC("SCRIPTRELOAD"),             EVT_SCRIPTRELOAD)
        .Const(_SC("COUNT"),                    EVT_COUNT)
    );

    Sqrat::ConstTable(vm).Enum(_SC("ECREATEREASON"), Sqrat::Enumeration(vm)
        .Const(_SC("DEFAULT"),      SQMOD_CREATE_DEFAULT)
        .Const(_SC("MANUAL"),       SQMOD_CREATE_MANUAL)
        .Const(_SC("POOL"),         SQMOD_CREATE_POOL)
        .Const(_SC("AUTOMATIC"),    SQMOD_CREATE_AUTOMATIC)
        .Const(_SC("OVERWRITE"),    SQMOD_CREATE_OVERWRITE)
        .Const(_SC("RESURECT"),     SQMOD_CREATE_RESURECT)
    );

    Sqrat::ConstTable(vm).Enum(_SC("EDESTROYREASON"), Sqrat::Enumeration(vm)
        .Const(_SC("DEFAULT"),      SQMOD_DESTROY_DEFAULT)
        .Const(_SC("MANUAL"),       SQMOD_DESTROY_MANUAL)
        .Const(_SC("POOL"),         SQMOD_DESTROY_POOL)
        .Const(_SC("AUTOMATIC"),    SQMOD_DESTROY_AUTOMATIC)
        .Const(_SC("OVERWRITE"),    SQMOD_DESTROY_OVERWRITE)
        .Const(_SC("CLEANUP"),      SQMOD_DESTROY_CLEANUP)
    );

    Sqrat::ConstTable(vm).Enum(_SC("EENTITYPOOLUPD"), Sqrat::Enumeration(vm)
        .Const(_SC("VEHICLE"),      SQMOD_ENTITY_POOL_VEHICLE)
        .Const(_SC("OBJECT"),       SQMOD_ENTITY_POOL_OBJECT)
        .Const(_SC("PICKUP"),       SQMOD_ENTITY_POOL_PICKUP)
        .Const(_SC("RADIO"),        SQMOD_ENTITY_POOL_RADIO)
        .Const(_SC("SPRITE"),       SQMOD_ENTITY_POOL_SPRITE)
        .Const(_SC("TEXTDRAW"),     SQMOD_ENTITY_POOL_TEXTDRAW)
        .Const(_SC("BLIP"),         SQMOD_ENTITY_POOL_BLIP)
    );

    Sqrat::ConstTable(vm).Enum(_SC("EVEHICLEUPD"), Sqrat::Enumeration(vm)
        .Const(_SC("DRIVER"),       SQMOD_VEHICLEUPD_DRIVER)
        .Const(_SC("OTHER"),        SQMOD_VEHICLEUPD_OTHER)
    );

    Sqrat::ConstTable(vm).Enum(_SC("EPLAYERUPD"), Sqrat::Enumeration(vm)
        .Const(_SC("ONFOOT"),       SQMOD_PLAYERUPD_ONFOOT)
        .Const(_SC("AIM"),          SQMOD_PLAYERUPD_AIM)
        .Const(_SC("DRIVER"),       SQMOD_PLAYERUPD_DRIVER)
        .Const(_SC("PASSENGER"),    SQMOD_PLAYERUPD_PASSENGER)
    );

    Sqrat::ConstTable(vm).Enum(_SC("EPARTREASON"), Sqrat::Enumeration(vm)
        .Const(_SC("TIMEOUT"),      SQMOD_PARTREASON_TIMEOUT)
        .Const(_SC("DISCONNECTED"), SQMOD_PARTREASON_DISCONNECTED)
        .Const(_SC("KICKEDBANNED"), SQMOD_PARTREASON_KICKEDBANNED)
        .Const(_SC("CRASHED"),      SQMOD_PARTREASON_CRASHED)
    );

    Sqrat::ConstTable(vm).Enum(_SC("EBODYPART"), Sqrat::Enumeration(vm)
        .Const(_SC("BODY"),         SQMOD_BODYPART_BODY)
        .Const(_SC("TORSO"),        SQMOD_BODYPART_TORSO)
        .Const(_SC("LEFTARM"),      SQMOD_BODYPART_LEFTARM)
        .Const(_SC("LARM"),         SQMOD_BODYPART_LEFTARM)
        .Const(_SC("RIGHTARM"),     SQMOD_BODYPART_RIGHTARM)
        .Const(_SC("RARM"),         SQMOD_BODYPART_RIGHTARM)
        .Const(_SC("LEFTLEG"),      SQMOD_BODYPART_LEFTLEG)
        .Const(_SC("LLEG"),         SQMOD_BODYPART_LEFTLEG)
        .Const(_SC("RIGHTLEG"),     SQMOD_BODYPART_RIGHTLEG)
        .Const(_SC("RLEG"),         SQMOD_BODYPART_RIGHTLEG)
        .Const(_SC("HEAD"),         SQMOD_BODYPART_HEAD)
    );

    Sqrat::ConstTable(vm).Enum(_SC("EPLAYERSTATE"), Sqrat::Enumeration(vm)
        .Const(_SC("NONE"),                     SQMOD_PLAYER_STATE_NONE)
        .Const(_SC("NORMAL"),                   SQMOD_PLAYER_STATE_NORMAL)
        .Const(_SC("SHOOTING"),                 SQMOD_PLAYER_STATE_SHOOTING)
        .Const(_SC("DRIVER"),                   SQMOD_PLAYER_STATE_DRIVER)
        .Const(_SC("PASSENGER"),                SQMOD_PLAYER_STATE_PASSENGER)
        .Const(_SC("ENTERING_AS_DRIVER"),       SQMOD_PLAYER_STATE_ENTERING_AS_DRIVER)
        .Const(_SC("ENTERING_AS_PASSENGER"),    SQMOD_PLAYER_STATE_ENTERING_AS_PASSENGER)
        .Const(_SC("EXITING_VEHICLE"),          SQMOD_PLAYER_STATE_EXITING_VEHICLE)
        .Const(_SC("UNSPAWNED"),                SQMOD_PLAYER_STATE_UNSPAWNED)
    );

    Sqrat::ConstTable(vm).Enum(_SC("EPLAYERACTION"), Sqrat::Enumeration(vm)
        .Const(_SC("NONE"),                     SQMOD_PLAYER_ACTION_NONE)
        .Const(_SC("NORMAL"),                   SQMOD_PLAYER_ACTION_NORMAL)
        .Const(_SC("AIMING"),                   SQMOD_PLAYER_ACTION_AIMING)
        .Const(_SC("SHOOTING"),                 SQMOD_PLAYER_ACTION_SHOOTING)
        .Const(_SC("JUMPING"),                  SQMOD_PLAYER_ACTION_JUMPING)
        .Const(_SC("LYING_ON_GROUND"),          SQMOD_PLAYER_ACTION_LYING_ON_GROUND)
        .Const(_SC("GETTING_UP"),               SQMOD_PLAYER_ACTION_GETTING_UP)
        .Const(_SC("JUMPING_FROM_VEHICLE"),     SQMOD_PLAYER_ACTION_JUMPING_FROM_VEHICLE)
        .Const(_SC("DRIVING"),                  SQMOD_PLAYER_ACTION_DRIVING)
        .Const(_SC("DYING"),                    SQMOD_PLAYER_ACTION_DYING)
        .Const(_SC("WASTED"),                   SQMOD_PLAYER_ACTION_WASTED)
        .Const(_SC("ENTERING_VEHICLE"),         SQMOD_PLAYER_ACTION_ENTERING_VEHICLE)
        .Const(_SC("EXITING_VEHICLE"),          SQMOD_PLAYER_ACTION_EXITING_VEHICLE)
    );

    Sqrat::ConstTable(vm).Enum(_SC("EWEATHER"), Sqrat::Enumeration(vm)
        .Const(_SC("MOSTLYCLEAR"),      SQMOD_WEATHER_MOSTLYCLEAR)
        .Const(_SC("OVERCAST"),         SQMOD_WEATHER_OVERCAST)
        .Const(_SC("THUNDERSTORM"),     SQMOD_WEATHER_THUNDERSTORM)
        .Const(_SC("STORM"),            SQMOD_WEATHER_STORM)
        .Const(_SC("STORMY"),           SQMOD_WEATHER_STORMY)
        .Const(_SC("FOGGY"),            SQMOD_WEATHER_FOGGY)
        .Const(_SC("FOG"),              SQMOD_WEATHER_FOG)
        .Const(_SC("CLEAR"),            SQMOD_WEATHER_CLEAR)
        .Const(_SC("SUNNY"),            SQMOD_WEATHER_SUNNY)
        .Const(_SC("RAIN"),             SQMOD_WEATHER_RAIN)
        .Const(_SC("RAINY"),            SQMOD_WEATHER_RAINY)
        .Const(_SC("DARKCLOUDY"),       SQMOD_WEATHER_DARKCLOUDY)
        .Const(_SC("LIGHTCLOUDY"),      SQMOD_WEATHER_LIGHTCLOUDY)
        .Const(_SC("OVERCASTCLOUDY"),   SQMOD_WEATHER_OVERCASTCLOUDY)
        .Const(_SC("BLACKCLOUDS"),      SQMOD_WEATHER_BLACKCLOUDS)
    );

    Sqrat::ConstTable(vm).Enum(_SC("EWEAPON"), Sqrat::Enumeration(vm)
        .Const(_SC("UNKNWON"),          SQMOD_UNKNOWN)
        .Const(_SC("UNARMED"),          SQMOD_WEAPON_UNARMED)
        .Const(_SC("BRASSKNUCKLES"),    SQMOD_WEAPON_BRASSKNUCKLES)
        .Const(_SC("SCREWDRIVER"),      SQMOD_WEAPON_SCREWDRIVER)
        .Const(_SC("GOLFCLUB"),         SQMOD_WEAPON_GOLFCLUB)
        .Const(_SC("NIGHTSTICK"),       SQMOD_WEAPON_NIGHTSTICK)
        .Const(_SC("KNIFE"),            SQMOD_WEAPON_KNIFE)
        .Const(_SC("BASEBALLBAT"),      SQMOD_WEAPON_BASEBALLBAT)
        .Const(_SC("HAMMER"),           SQMOD_WEAPON_HAMMER)
        .Const(_SC("MEATCLEAVER"),      SQMOD_WEAPON_MEATCLEAVER)
        .Const(_SC("MACHETE"),          SQMOD_WEAPON_MACHETE)
        .Const(_SC("KATANA"),           SQMOD_WEAPON_KATANA)
        .Const(_SC("CHAINSAW"),         SQMOD_WEAPON_CHAINSAW)
        .Const(_SC("GRENADE"),          SQMOD_WEAPON_GRENADE)
        .Const(_SC("REMOTE"),           SQMOD_WEAPON_REMOTE)
        .Const(_SC("TEARGAS"),          SQMOD_WEAPON_TEARGAS)
        .Const(_SC("MOLOTOV"),          SQMOD_WEAPON_MOLOTOV)
        .Const(_SC("ROCKET"),           SQMOD_WEAPON_ROCKET)
        .Const(_SC("COLT45"),           SQMOD_WEAPON_COLT45)
        .Const(_SC("PYTHON"),           SQMOD_WEAPON_PYTHON)
        .Const(_SC("SHOTGUN"),          SQMOD_WEAPON_SHOTGUN)
        .Const(_SC("SPAS12"),           SQMOD_WEAPON_SPAS12)
        .Const(_SC("STUBBY"),           SQMOD_WEAPON_STUBBY)
        .Const(_SC("TEC9"),             SQMOD_WEAPON_TEC9)
        .Const(_SC("UZI"),              SQMOD_WEAPON_UZI)
        .Const(_SC("INGRAM"),           SQMOD_WEAPON_INGRAM)
        .Const(_SC("MP5"),              SQMOD_WEAPON_MP5)
        .Const(_SC("M4"),               SQMOD_WEAPON_M4)
        .Const(_SC("RUGER"),            SQMOD_WEAPON_RUGER)
        .Const(_SC("SNIPER"),           SQMOD_WEAPON_SNIPER)
        .Const(_SC("LASERSCOPE"),       SQMOD_WEAPON_LASERSCOPE)
        .Const(_SC("ROCKETLAUNCHER"),   SQMOD_WEAPON_ROCKETLAUNCHER)
        .Const(_SC("FLAMETHROWER"),     SQMOD_WEAPON_FLAMETHROWER)
        .Const(_SC("M60"),              SQMOD_WEAPON_M60)
        .Const(_SC("MINIGUN"),          SQMOD_WEAPON_MINIGUN)
        .Const(_SC("BOMB"),             SQMOD_WEAPON_BOMB)
        .Const(_SC("HELICANNON"),       SQMOD_WEAPON_HELICANNON)
        .Const(_SC("CAMERA"),           SQMOD_WEAPON_CAMERA)
        .Const(_SC("VEHICLE"),          SQMOD_WEAPON_VEHICLE)
        .Const(_SC("EXPLOSION1"),       SQMOD_WEAPON_EXPLOSION1)
        .Const(_SC("DRIVEBY"),          SQMOD_WEAPON_DRIVEBY)
        .Const(_SC("DROWNED"),          SQMOD_WEAPON_DROWNED)
        .Const(_SC("FALL"),             SQMOD_WEAPON_FALL)
        .Const(_SC("EXPLOSION2"),       SQMOD_WEAPON_EXPLOSION2)
        .Const(_SC("SUICIDE"),          SQMOD_WEAPON_SUICIDE)
    );

    Sqrat::ConstTable(vm).Enum(_SC("EVEHICLE"), Sqrat::Enumeration(vm)
        .Const(_SC("UNKNOWN"),          SQMOD_UNKNOWN)
        .Const(_SC("LANDSTALKER"),      SQMOD_VEHICLE_LANDSTALKER)
        .Const(_SC("IDAHO"),            SQMOD_VEHICLE_IDAHO)
        .Const(_SC("STINGER"),          SQMOD_VEHICLE_STINGER)
        .Const(_SC("LINERUNNER"),       SQMOD_VEHICLE_LINERUNNER)
        .Const(_SC("PERENNIAL"),        SQMOD_VEHICLE_PERENNIAL)
        .Const(_SC("SENTINEL"),         SQMOD_VEHICLE_SENTINEL)
        .Const(_SC("RIO"),              SQMOD_VEHICLE_RIO)
        .Const(_SC("FIRETRUCK"),        SQMOD_VEHICLE_FIRETRUCK)
        .Const(_SC("TRASHMASTER"),      SQMOD_VEHICLE_TRASHMASTER)
        .Const(_SC("STRETCH"),          SQMOD_VEHICLE_STRETCH)
        .Const(_SC("MANANA"),           SQMOD_VEHICLE_MANANA)
        .Const(_SC("INFERNUS"),         SQMOD_VEHICLE_INFERNUS)
        .Const(_SC("VOODOO"),           SQMOD_VEHICLE_VOODOO)
        .Const(_SC("PONY"),             SQMOD_VEHICLE_PONY)
        .Const(_SC("MULE"),             SQMOD_VEHICLE_MULE)
        .Const(_SC("CHEETAH"),          SQMOD_VEHICLE_CHEETAH)
        .Const(_SC("AMBULANCE"),        SQMOD_VEHICLE_AMBULANCE)
        .Const(_SC("FBIWASHINGTON"),    SQMOD_VEHICLE_FBIWASHINGTON)
        .Const(_SC("MOONBEAM"),         SQMOD_VEHICLE_MOONBEAM)
        .Const(_SC("ESPERANTO"),        SQMOD_VEHICLE_ESPERANTO)
        .Const(_SC("TAXI"),             SQMOD_VEHICLE_TAXI)
        .Const(_SC("WASHINGTON"),       SQMOD_VEHICLE_WASHINGTON)
        .Const(_SC("BOBCAT"),           SQMOD_VEHICLE_BOBCAT)
        .Const(_SC("MRWHOOPEE"),        SQMOD_VEHICLE_MRWHOOPEE)
        .Const(_SC("BFINJECTION"),      SQMOD_VEHICLE_BFINJECTION)
        .Const(_SC("HUNTER"),           SQMOD_VEHICLE_HUNTER)
        .Const(_SC("POLICE"),           SQMOD_VEHICLE_POLICE)
        .Const(_SC("ENFORCER"),         SQMOD_VEHICLE_ENFORCER)
        .Const(_SC("SECURICAR"),        SQMOD_VEHICLE_SECURICAR)
        .Const(_SC("BANSHEE"),          SQMOD_VEHICLE_BANSHEE)
        .Const(_SC("PREDATOR"),         SQMOD_VEHICLE_PREDATOR)
        .Const(_SC("BUS"),              SQMOD_VEHICLE_BUS)
        .Const(_SC("RHINO"),            SQMOD_VEHICLE_RHINO)
        .Const(_SC("BARRACKSOL"),       SQMOD_VEHICLE_BARRACKSOL)
        .Const(_SC("BARRACKS"),         SQMOD_VEHICLE_BARRACKS)
        .Const(_SC("CUBANHERMES"),      SQMOD_VEHICLE_CUBANHERMES)
        .Const(_SC("HELICOPTER"),       SQMOD_VEHICLE_HELICOPTER)
        .Const(_SC("ANGEL"),            SQMOD_VEHICLE_ANGEL)
        .Const(_SC("COACH"),            SQMOD_VEHICLE_COACH)
        .Const(_SC("CABBIE"),           SQMOD_VEHICLE_CABBIE)
        .Const(_SC("STALLION"),         SQMOD_VEHICLE_STALLION)
        .Const(_SC("RUMPO"),            SQMOD_VEHICLE_RUMPO)
        .Const(_SC("RCBANDIT"),         SQMOD_VEHICLE_RCBANDIT)
        .Const(_SC("HEARSE"),           SQMOD_VEHICLE_HEARSE)
        .Const(_SC("PACKER"),           SQMOD_VEHICLE_PACKER)
        .Const(_SC("SENTINELXS"),       SQMOD_VEHICLE_SENTINELXS)
        .Const(_SC("ADMIRAL"),          SQMOD_VEHICLE_ADMIRAL)
        .Const(_SC("SQUALO"),           SQMOD_VEHICLE_SQUALO)
        .Const(_SC("SEASPARROW"),       SQMOD_VEHICLE_SEASPARROW)
        .Const(_SC("PIZZABOY"),         SQMOD_VEHICLE_PIZZABOY)
        .Const(_SC("GANGBURRITO"),      SQMOD_VEHICLE_GANGBURRITO)
        .Const(_SC("AIRTRAIN"),         SQMOD_VEHICLE_AIRTRAIN)
        .Const(_SC("DEADDODO"),         SQMOD_VEHICLE_DEADDODO)
        .Const(_SC("SPEEDER"),          SQMOD_VEHICLE_SPEEDER)
        .Const(_SC("REEFER"),           SQMOD_VEHICLE_REEFER)
        .Const(_SC("TROPIC"),           SQMOD_VEHICLE_TROPIC)
        .Const(_SC("FLATBED"),          SQMOD_VEHICLE_FLATBED)
        .Const(_SC("YANKEE"),           SQMOD_VEHICLE_YANKEE)
        .Const(_SC("CADDY"),            SQMOD_VEHICLE_CADDY)
        .Const(_SC("ZEBRA"),            SQMOD_VEHICLE_ZEBRA)
        .Const(_SC("ZEBRACAB"),         SQMOD_VEHICLE_ZEBRACAB)
        .Const(_SC("TOPFUN"),           SQMOD_VEHICLE_TOPFUN)
        .Const(_SC("SKIMMER"),          SQMOD_VEHICLE_SKIMMER)
        .Const(_SC("PCJ600"),           SQMOD_VEHICLE_PCJ600)
        .Const(_SC("PCJ"),              SQMOD_VEHICLE_PCJ)
        .Const(_SC("FAGGIO"),           SQMOD_VEHICLE_FAGGIO)
        .Const(_SC("FREEWAY"),          SQMOD_VEHICLE_FREEWAY)
        .Const(_SC("RCBARON"),          SQMOD_VEHICLE_RCBARON)
        .Const(_SC("RCRAIDER"),         SQMOD_VEHICLE_RCRAIDER)
        .Const(_SC("GLENDALE"),         SQMOD_VEHICLE_GLENDALE)
        .Const(_SC("OCEANIC"),          SQMOD_VEHICLE_OCEANIC)
        .Const(_SC("SANCHEZ"),          SQMOD_VEHICLE_SANCHEZ)
        .Const(_SC("SPARROW"),          SQMOD_VEHICLE_SPARROW)
        .Const(_SC("PATRIOT"),          SQMOD_VEHICLE_PATRIOT)
        .Const(_SC("LOVEFIST"),         SQMOD_VEHICLE_LOVEFIST)
        .Const(_SC("COASTGUARD"),       SQMOD_VEHICLE_COASTGUARD)
        .Const(_SC("DINGHY"),           SQMOD_VEHICLE_DINGHY)
        .Const(_SC("HERMES"),           SQMOD_VEHICLE_HERMES)
        .Const(_SC("SABRE"),            SQMOD_VEHICLE_SABRE)
        .Const(_SC("SABRETURBO"),       SQMOD_VEHICLE_SABRETURBO)
        .Const(_SC("PHOENIX"),          SQMOD_VEHICLE_PHOENIX)
        .Const(_SC("WALTON"),           SQMOD_VEHICLE_WALTON)
        .Const(_SC("REGINA"),           SQMOD_VEHICLE_REGINA)
        .Const(_SC("COMET"),            SQMOD_VEHICLE_COMET)
        .Const(_SC("DELUXO"),           SQMOD_VEHICLE_DELUXO)
        .Const(_SC("BURRITO"),          SQMOD_VEHICLE_BURRITO)
        .Const(_SC("SPANDEX"),          SQMOD_VEHICLE_SPANDEX)
        .Const(_SC("SPANDEXPRESS"),     SQMOD_VEHICLE_SPANDEXPRESS)
        .Const(_SC("MARQUIS"),          SQMOD_VEHICLE_MARQUIS)
        .Const(_SC("BAGGAGE"),          SQMOD_VEHICLE_BAGGAGE)
        .Const(_SC("BAGGAGEHANDLER"),   SQMOD_VEHICLE_BAGGAGEHANDLER)
        .Const(_SC("KAUFMAN"),          SQMOD_VEHICLE_KAUFMAN)
        .Const(_SC("KAUFMANCAB"),       SQMOD_VEHICLE_KAUFMANCAB)
        .Const(_SC("MAVERICK"),         SQMOD_VEHICLE_MAVERICK)
        .Const(_SC("VCNMAVERICK"),      SQMOD_VEHICLE_VCNMAVERICK)
        .Const(_SC("RANCHER"),          SQMOD_VEHICLE_RANCHER)
        .Const(_SC("FBIRANCHER"),       SQMOD_VEHICLE_FBIRANCHER)
        .Const(_SC("VIRGO"),            SQMOD_VEHICLE_VIRGO)
        .Const(_SC("GREENWOOD"),        SQMOD_VEHICLE_GREENWOOD)
        .Const(_SC("CUBANJETMAX"),      SQMOD_VEHICLE_CUBANJETMAX)
        .Const(_SC("HOTRING1"),         SQMOD_VEHICLE_HOTRING1)
        .Const(_SC("HOTRINGRACER1"),    SQMOD_VEHICLE_HOTRINGRACER1)
        .Const(_SC("SANDKING"),         SQMOD_VEHICLE_SANDKING)
        .Const(_SC("BLISTA"),           SQMOD_VEHICLE_BLISTA)
        .Const(_SC("BLISTAC"),          SQMOD_VEHICLE_BLISTAC)
        .Const(_SC("BLISTACOMPACT"),    SQMOD_VEHICLE_BLISTACOMPACT)
        .Const(_SC("COMPACT"),          SQMOD_VEHICLE_COMPACT)
        .Const(_SC("POLICEMAV"),        SQMOD_VEHICLE_POLICEMAV)
        .Const(_SC("POLICEMAVERICK"),   SQMOD_VEHICLE_POLICEMAVERICK)
        .Const(_SC("BOXVILLE"),         SQMOD_VEHICLE_BOXVILLE)
        .Const(_SC("BENSON"),           SQMOD_VEHICLE_BENSON)
        .Const(_SC("MESA"),             SQMOD_VEHICLE_MESA)
        .Const(_SC("MESAGRANDE"),       SQMOD_VEHICLE_MESAGRANDE)
        .Const(_SC("RCGOBLIN"),         SQMOD_VEHICLE_RCGOBLIN)
        .Const(_SC("HOTRING2"),         SQMOD_VEHICLE_HOTRING2)
        .Const(_SC("HOTRINGRACER2"),    SQMOD_VEHICLE_HOTRINGRACER2)
        .Const(_SC("HOTRING3"),         SQMOD_VEHICLE_HOTRING3)
        .Const(_SC("HOTRINGRACER3"),    SQMOD_VEHICLE_HOTRINGRACER3)
        .Const(_SC("BLOODRING1"),       SQMOD_VEHICLE_BLOODRING1)
        .Const(_SC("BLOODRINGBANGER1"), SQMOD_VEHICLE_BLOODRINGBANGER1)
        .Const(_SC("BLOODRING2"),       SQMOD_VEHICLE_BLOODRING2)
        .Const(_SC("BLOODRINGBANGER2"), SQMOD_VEHICLE_BLOODRINGBANGER2)
        .Const(_SC("VICECHEE"),         SQMOD_VEHICLE_VICECHEE)
        .Const(_SC("POLICECHEETAH"),    SQMOD_VEHICLE_POLICECHEETAH)
        .Const(_SC("FBICHEETAH"),       SQMOD_VEHICLE_FBICHEETAH)
        .Const(_SC("CHEETAH2"),         SQMOD_VEHICLE_CHEETAH2)
    );

    Sqrat::ConstTable(vm).Enum(_SC("ESKIN"), Sqrat::Enumeration(vm)
        .Const(_SC("UNKNOWN"),                  SQMOD_UNKNOWN)
        .Const(_SC("TOMMY_VERCETTI"),           SQMOD_SKIN_TOMMY_VERCETTI)
        .Const(_SC("COP"),                      SQMOD_SKIN_COP)
        .Const(_SC("SWAT"),                     SQMOD_SKIN_SWAT)
        .Const(_SC("FBI"),                      SQMOD_SKIN_FBI)
        .Const(_SC("ARMY"),                     SQMOD_SKIN_ARMY)
        .Const(_SC("PARAMEDIC"),                SQMOD_SKIN_PARAMEDIC)
        .Const(_SC("FIREMAN"),                  SQMOD_SKIN_FIREMAN)
        .Const(_SC("GOLF_GUY_A"),               SQMOD_SKIN_GOLF_GUY_A)
        .Const(_SC("BUM_LADY_A"),               SQMOD_SKIN_BUM_LADY_A)
        .Const(_SC("BUM_LADY_B"),               SQMOD_SKIN_BUM_LADY_B)
        .Const(_SC("PUNK_A"),                   SQMOD_SKIN_PUNK_A)
        .Const(_SC("LAWYER"),                   SQMOD_SKIN_LAWYER)
        .Const(_SC("SPANISH_LADY_A"),           SQMOD_SKIN_SPANISH_LADY_A)
        .Const(_SC("SPANISH_LADY_B"),           SQMOD_SKIN_SPANISH_LADY_B)
        .Const(_SC("COOL_GUY_A"),               SQMOD_SKIN_COOL_GUY_A)
        .Const(_SC("ARABIC_GUY"),               SQMOD_SKIN_ARABIC_GUY)
        .Const(_SC("BEACH_LADY_A"),             SQMOD_SKIN_BEACH_LADY_A)
        .Const(_SC("BEACH_LADY_B"),             SQMOD_SKIN_BEACH_LADY_B)
        .Const(_SC("BEACH_GUY_A"),              SQMOD_SKIN_BEACH_GUY_A)
        .Const(_SC("BEACH_GUY_B"),              SQMOD_SKIN_BEACH_GUY_B)
        .Const(_SC("OFFICE_LADY_A"),            SQMOD_SKIN_OFFICE_LADY_A)
        .Const(_SC("WAITRESS_A"),               SQMOD_SKIN_WAITRESS_A)
        .Const(_SC("FOOD_LADY"),                SQMOD_SKIN_FOOD_LADY)
        .Const(_SC("PROSTITUTE_A"),             SQMOD_SKIN_PROSTITUTE_A)
        .Const(_SC("BUM_LADY_C"),               SQMOD_SKIN_BUM_LADY_C)
        .Const(_SC("BUM_GUY_A"),                SQMOD_SKIN_BUM_GUY_A)
        .Const(_SC("GARBAGEMAN_A"),             SQMOD_SKIN_GARBAGEMAN_A)
        .Const(_SC("TAXI_DRIVER_A"),            SQMOD_SKIN_TAXI_DRIVER_A)
        .Const(_SC("HATIAN_A"),                 SQMOD_SKIN_HATIAN_A)
        .Const(_SC("CRIMINAL_A"),               SQMOD_SKIN_CRIMINAL_A)
        .Const(_SC("HOOD_LADY"),                SQMOD_SKIN_HOOD_LADY)
        .Const(_SC("GRANNY_A"),                 SQMOD_SKIN_GRANNY_A)
        .Const(_SC("BUSINESS_MAN_A"),           SQMOD_SKIN_BUSINESS_MAN_A)
        .Const(_SC("CHURCH_GUY"),               SQMOD_SKIN_CHURCH_GUY)
        .Const(_SC("CLUB_LADY"),                SQMOD_SKIN_CLUB_LADY)
        .Const(_SC("CHURCH_LADY"),              SQMOD_SKIN_CHURCH_LADY)
        .Const(_SC("PIMP"),                     SQMOD_SKIN_PIMP)
        .Const(_SC("BEACH_LADY_C"),             SQMOD_SKIN_BEACH_LADY_C)
        .Const(_SC("BEACH_GUY_C"),              SQMOD_SKIN_BEACH_GUY_C)
        .Const(_SC("BEACH_LADY_D"),             SQMOD_SKIN_BEACH_LADY_D)
        .Const(_SC("BEACH_GUY_D"),              SQMOD_SKIN_BEACH_GUY_D)
        .Const(_SC("BUSINESS_MAN_B"),           SQMOD_SKIN_BUSINESS_MAN_B)
        .Const(_SC("PROSTITUTE_B"),             SQMOD_SKIN_PROSTITUTE_B)
        .Const(_SC("BUM_LADY_D"),               SQMOD_SKIN_BUM_LADY_D)
        .Const(_SC("BUM_GUY_B"),                SQMOD_SKIN_BUM_GUY_B)
        .Const(_SC("HATIAN_B"),                 SQMOD_SKIN_HATIAN_B)
        .Const(_SC("CONSTRUCTION_WORKER_A"),    SQMOD_SKIN_CONSTRUCTION_WORKER_A)
        .Const(_SC("PUNK_B"),                   SQMOD_SKIN_PUNK_B)
        .Const(_SC("PROSTITUTE_C"),             SQMOD_SKIN_PROSTITUTE_C)
        .Const(_SC("GRANNY_B"),                 SQMOD_SKIN_GRANNY_B)
        .Const(_SC("PUNK_C"),                   SQMOD_SKIN_PUNK_C)
        .Const(_SC("BUSINESS_MAN_C"),           SQMOD_SKIN_BUSINESS_MAN_C)
        .Const(_SC("SPANISH_LADY_C"),           SQMOD_SKIN_SPANISH_LADY_C)
        .Const(_SC("SPANISH_LADY_D"),           SQMOD_SKIN_SPANISH_LADY_D)
        .Const(_SC("COOL_GUY_B"),               SQMOD_SKIN_COOL_GUY_B)
        .Const(_SC("BUSINESS_MAN_D"),           SQMOD_SKIN_BUSINESS_MAN_D)
        .Const(_SC("BEACH_LADY_E"),             SQMOD_SKIN_BEACH_LADY_E)
        .Const(_SC("BEACH_GUY_E"),              SQMOD_SKIN_BEACH_GUY_E)
        .Const(_SC("BEACH_LADY_F"),             SQMOD_SKIN_BEACH_LADY_F)
        .Const(_SC("BEACH_GUY_F"),              SQMOD_SKIN_BEACH_GUY_F)
        .Const(_SC("CONSTRUCTION_WORKER_B"),    SQMOD_SKIN_CONSTRUCTION_WORKER_B)
        .Const(_SC("GOLF_GUY_B"),               SQMOD_SKIN_GOLF_GUY_B)
        .Const(_SC("GOLF_LADY"),                SQMOD_SKIN_GOLF_LADY)
        .Const(_SC("GOLF_GUY_C"),               SQMOD_SKIN_GOLF_GUY_C)
        .Const(_SC("BEACH_LADY_G"),             SQMOD_SKIN_BEACH_LADY_G)
        .Const(_SC("BEACH_GUY_G"),              SQMOD_SKIN_BEACH_GUY_G)
        .Const(_SC("OFFICE_LADY_B"),            SQMOD_SKIN_OFFICE_LADY_B)
        .Const(_SC("BUSINESS_MAN_E"),           SQMOD_SKIN_BUSINESS_MAN_E)
        .Const(_SC("BUSINESS_MAN_F"),           SQMOD_SKIN_BUSINESS_MAN_F)
        .Const(_SC("PROSTITUTE_D"),             SQMOD_SKIN_PROSTITUTE_D)
        .Const(_SC("BUM_LADY_E"),               SQMOD_SKIN_BUM_LADY_E)
        .Const(_SC("BUM_GUY_C"),                SQMOD_SKIN_BUM_GUY_C)
        .Const(_SC("SPANISH_GUY"),              SQMOD_SKIN_SPANISH_GUY)
        .Const(_SC("TAXI_DRIVER_B"),            SQMOD_SKIN_TAXI_DRIVER_B)
        .Const(_SC("GYM_LADY"),                 SQMOD_SKIN_GYM_LADY)
        .Const(_SC("GYM_GUY"),                  SQMOD_SKIN_GYM_GUY)
        .Const(_SC("SKATE_LADY"),               SQMOD_SKIN_SKATE_LADY)
        .Const(_SC("SKATE_GUY"),                SQMOD_SKIN_SKATE_GUY)
        .Const(_SC("SHOPPER_A"),                SQMOD_SKIN_SHOPPER_A)
        .Const(_SC("SHOPPER_B"),                SQMOD_SKIN_SHOPPER_B)
        .Const(_SC("TOURIST_A"),                SQMOD_SKIN_TOURIST_A)
        .Const(_SC("TOURIST_B"),                SQMOD_SKIN_TOURIST_B)
        .Const(_SC("CUBAN_A"),                  SQMOD_SKIN_CUBAN_A)
        .Const(_SC("CUBAN_B"),                  SQMOD_SKIN_CUBAN_B)
        .Const(_SC("HATIAN_C"),                 SQMOD_SKIN_HATIAN_C)
        .Const(_SC("HATIAN_D"),                 SQMOD_SKIN_HATIAN_D)
        .Const(_SC("SHARK_A"),                  SQMOD_SKIN_SHARK_A)
        .Const(_SC("SHARK_B"),                  SQMOD_SKIN_SHARK_B)
        .Const(_SC("DIAZ_GUY_A"),               SQMOD_SKIN_DIAZ_GUY_A)
        .Const(_SC("DIAZ_GUY_B"),               SQMOD_SKIN_DIAZ_GUY_B)
        .Const(_SC("DBP_SECURITY_A"),           SQMOD_SKIN_DBP_SECURITY_A)
        .Const(_SC("DBP_SECURITY_B"),           SQMOD_SKIN_DBP_SECURITY_B)
        .Const(_SC("BIKER_A"),                  SQMOD_SKIN_BIKER_A)
        .Const(_SC("BIKER_B"),                  SQMOD_SKIN_BIKER_B)
        .Const(_SC("VERCETTI_GUY_A"),           SQMOD_SKIN_VERCETTI_GUY_A)
        .Const(_SC("VERCETTI_GUY_B"),           SQMOD_SKIN_VERCETTI_GUY_B)
        .Const(_SC("UNDERCOVER_COP_A"),         SQMOD_SKIN_UNDERCOVER_COP_A)
        .Const(_SC("UNDERCOVER_COP_B"),         SQMOD_SKIN_UNDERCOVER_COP_B)
        .Const(_SC("UNDERCOVER_COP_C"),         SQMOD_SKIN_UNDERCOVER_COP_C)
        .Const(_SC("UNDERCOVER_COP_D"),         SQMOD_SKIN_UNDERCOVER_COP_D)
        .Const(_SC("UNDERCOVER_COP_E"),         SQMOD_SKIN_UNDERCOVER_COP_E)
        .Const(_SC("UNDERCOVER_COP_F"),         SQMOD_SKIN_UNDERCOVER_COP_F)
        .Const(_SC("RICH_GUY"),                 SQMOD_SKIN_RICH_GUY)
        .Const(_SC("COOL_GUY_C"),               SQMOD_SKIN_COOL_GUY_C)
        .Const(_SC("PROSTITUTE_E"),             SQMOD_SKIN_PROSTITUTE_E)
        .Const(_SC("PROSTITUTE_F"),             SQMOD_SKIN_PROSTITUTE_F)
        .Const(_SC("LOVE_FIST_A"),              SQMOD_SKIN_LOVE_FIST_A)
        .Const(_SC("KEN_ROSENBURG"),            SQMOD_SKIN_KEN_ROSENBURG)
        .Const(_SC("CANDY_SUXX"),               SQMOD_SKIN_CANDY_SUXX)
        .Const(_SC("HILARY"),                   SQMOD_SKIN_HILARY)
        .Const(_SC("LOVE_FIST_B"),              SQMOD_SKIN_LOVE_FIST_B)
        .Const(_SC("PHIL"),                     SQMOD_SKIN_PHIL)
        .Const(_SC("ROCKSTAR_GUY"),             SQMOD_SKIN_ROCKSTAR_GUY)
        .Const(_SC("SONNY"),                    SQMOD_SKIN_SONNY)
        .Const(_SC("LANCE_A"),                  SQMOD_SKIN_LANCE_A)
        .Const(_SC("MERCADES_A"),               SQMOD_SKIN_MERCADES_A)
        .Const(_SC("LOVE_FIST_C"),              SQMOD_SKIN_LOVE_FIST_C)
        .Const(_SC("ALEX_SRUB"),                SQMOD_SKIN_ALEX_SRUB)
        .Const(_SC("LANCE_COP"),                SQMOD_SKIN_LANCE_COP)
        .Const(_SC("LANCE_B"),                  SQMOD_SKIN_LANCE_B)
        .Const(_SC("CORTEZ"),                   SQMOD_SKIN_CORTEZ)
        .Const(_SC("LOVE_FIST_D"),              SQMOD_SKIN_LOVE_FIST_D)
        .Const(_SC("COLUMBIAN_GUY_A"),          SQMOD_SKIN_COLUMBIAN_GUY_A)
        .Const(_SC("HILARY_ROBBER"),            SQMOD_SKIN_HILARY_ROBBER)
        .Const(_SC("MERCADES_B"),               SQMOD_SKIN_MERCADES_B)
        .Const(_SC("CAM"),                      SQMOD_SKIN_CAM)
        .Const(_SC("CAM_ROBBER"),               SQMOD_SKIN_CAM_ROBBER)
        .Const(_SC("PHIL_ONE_ARM"),             SQMOD_SKIN_PHIL_ONE_ARM)
        .Const(_SC("PHIL_ROBBER"),              SQMOD_SKIN_PHIL_ROBBER)
        .Const(_SC("COOL_GUY_D"),               SQMOD_SKIN_COOL_GUY_D)
        .Const(_SC("PIZZAMAN"),                 SQMOD_SKIN_PIZZAMAN)
        .Const(_SC("TAXI_DRIVER_C"),            SQMOD_SKIN_TAXI_DRIVER_C)
        .Const(_SC("TAXI_DRIVER_D"),            SQMOD_SKIN_TAXI_DRIVER_D)
        .Const(_SC("SAILOR_A"),                 SQMOD_SKIN_SAILOR_A)
        .Const(_SC("SAILOR_B"),                 SQMOD_SKIN_SAILOR_B)
        .Const(_SC("SAILOR_C"),                 SQMOD_SKIN_SAILOR_C)
        .Const(_SC("CHEF"),                     SQMOD_SKIN_CHEF)
        .Const(_SC("CRIMINAL_B"),               SQMOD_SKIN_CRIMINAL_B)
        .Const(_SC("FRENCH_GUY"),               SQMOD_SKIN_FRENCH_GUY)
        .Const(_SC("GARBAGEMAN_B"),             SQMOD_SKIN_GARBAGEMAN_B)
        .Const(_SC("HATIAN_E"),                 SQMOD_SKIN_HATIAN_E)
        .Const(_SC("WAITRESS_B"),               SQMOD_SKIN_WAITRESS_B)
        .Const(_SC("SONNY_GUY_A"),              SQMOD_SKIN_SONNY_GUY_A)
        .Const(_SC("SONNY_GUY_B"),              SQMOD_SKIN_SONNY_GUY_B)
        .Const(_SC("SONNY_GUY_C"),              SQMOD_SKIN_SONNY_GUY_C)
        .Const(_SC("COLUMBIAN_GUY_B"),          SQMOD_SKIN_COLUMBIAN_GUY_B)
        .Const(_SC("THUG_A"),                   SQMOD_SKIN_THUG_A)
        .Const(_SC("BEACH_GUY_H"),              SQMOD_SKIN_BEACH_GUY_H)
        .Const(_SC("GARBAGEMAN_C"),             SQMOD_SKIN_GARBAGEMAN_C)
        .Const(_SC("GARBAGEMAN_D"),             SQMOD_SKIN_GARBAGEMAN_D)
        .Const(_SC("GARBAGEMAN_E"),             SQMOD_SKIN_GARBAGEMAN_E)
        .Const(_SC("TRANNY"),                   SQMOD_SKIN_TRANNY)
        .Const(_SC("THUG_B"),                   SQMOD_SKIN_THUG_B)
        .Const(_SC("SPANDEX_GUY_A"),            SQMOD_SKIN_SPANDEX_GUY_A)
        .Const(_SC("SPANDEX_GUY_B"),            SQMOD_SKIN_SPANDEX_GUY_B)
        .Const(_SC("STRIPPER_A"),               SQMOD_SKIN_STRIPPER_A)
        .Const(_SC("STRIPPER_B"),               SQMOD_SKIN_STRIPPER_B)
        .Const(_SC("STRIPPER_C"),               SQMOD_SKIN_STRIPPER_C)
        .Const(_SC("STORE_CLERK"),              SQMOD_SKIN_STORE_CLERK)
    );

    Sqrat::ConstTable(vm).Enum(_SC("EKEYCODE"), Sqrat::Enumeration(vm)
        .Const(_SC("ABNT_C1"),              SQMOD_KEYCODE_ABNT_C1)
        .Const(_SC("ABNT_C2"),              SQMOD_KEYCODE_ABNT_C2)
        .Const(_SC("ADD"),                  SQMOD_KEYCODE_ADD)
        .Const(_SC("ATTN"),                 SQMOD_KEYCODE_ATTN)
        .Const(_SC("BACK"),                 SQMOD_KEYCODE_BACK)
        .Const(_SC("CANCEL"),               SQMOD_KEYCODE_CANCEL)
        .Const(_SC("CLEAR"),                SQMOD_KEYCODE_CLEAR)
        .Const(_SC("CRSEL"),                SQMOD_KEYCODE_CRSEL)
        .Const(_SC("DECIMAL"),              SQMOD_KEYCODE_DECIMAL)
        .Const(_SC("DIVIDE"),               SQMOD_KEYCODE_DIVIDE)
        .Const(_SC("EREOF"),                SQMOD_KEYCODE_EREOF)
        .Const(_SC("ESCAPE"),               SQMOD_KEYCODE_ESCAPE)
        .Const(_SC("EXECUTE"),              SQMOD_KEYCODE_EXECUTE)
        .Const(_SC("EXSEL"),                SQMOD_KEYCODE_EXSEL)
        .Const(_SC("ICO_CLEAR"),            SQMOD_KEYCODE_ICO_CLEAR)
        .Const(_SC("ICO_HELP"),             SQMOD_KEYCODE_ICO_HELP)
        .Const(_SC("KEY_0"),                SQMOD_KEYCODE_KEY_0)
        .Const(_SC("KEY_1"),                SQMOD_KEYCODE_KEY_1)
        .Const(_SC("KEY_2"),                SQMOD_KEYCODE_KEY_2)
        .Const(_SC("KEY_3"),                SQMOD_KEYCODE_KEY_3)
        .Const(_SC("KEY_4"),                SQMOD_KEYCODE_KEY_4)
        .Const(_SC("KEY_5"),                SQMOD_KEYCODE_KEY_5)
        .Const(_SC("KEY_6"),                SQMOD_KEYCODE_KEY_6)
        .Const(_SC("KEY_7"),                SQMOD_KEYCODE_KEY_7)
        .Const(_SC("KEY_8"),                SQMOD_KEYCODE_KEY_8)
        .Const(_SC("KEY_9"),                SQMOD_KEYCODE_KEY_9)
        .Const(_SC("KEY_A"),                SQMOD_KEYCODE_KEY_A)
        .Const(_SC("KEY_B"),                SQMOD_KEYCODE_KEY_B)
        .Const(_SC("KEY_C"),                SQMOD_KEYCODE_KEY_C)
        .Const(_SC("KEY_D"),                SQMOD_KEYCODE_KEY_D)
        .Const(_SC("KEY_E"),                SQMOD_KEYCODE_KEY_E)
        .Const(_SC("KEY_F"),                SQMOD_KEYCODE_KEY_F)
        .Const(_SC("KEY_G"),                SQMOD_KEYCODE_KEY_G)
        .Const(_SC("KEY_H"),                SQMOD_KEYCODE_KEY_H)
        .Const(_SC("KEY_I"),                SQMOD_KEYCODE_KEY_I)
        .Const(_SC("KEY_J"),                SQMOD_KEYCODE_KEY_J)
        .Const(_SC("KEY_K"),                SQMOD_KEYCODE_KEY_K)
        .Const(_SC("KEY_L"),                SQMOD_KEYCODE_KEY_L)
        .Const(_SC("KEY_M"),                SQMOD_KEYCODE_KEY_M)
        .Const(_SC("KEY_N"),                SQMOD_KEYCODE_KEY_N)
        .Const(_SC("KEY_O"),                SQMOD_KEYCODE_KEY_O)
        .Const(_SC("KEY_P"),                SQMOD_KEYCODE_KEY_P)
        .Const(_SC("KEY_Q"),                SQMOD_KEYCODE_KEY_Q)
        .Const(_SC("KEY_R"),                SQMOD_KEYCODE_KEY_R)
        .Const(_SC("KEY_S"),                SQMOD_KEYCODE_KEY_S)
        .Const(_SC("KEY_T"),                SQMOD_KEYCODE_KEY_T)
        .Const(_SC("KEY_U"),                SQMOD_KEYCODE_KEY_U)
        .Const(_SC("KEY_V"),                SQMOD_KEYCODE_KEY_V)
        .Const(_SC("KEY_W"),                SQMOD_KEYCODE_KEY_W)
        .Const(_SC("KEY_X"),                SQMOD_KEYCODE_KEY_X)
        .Const(_SC("KEY_Y"),                SQMOD_KEYCODE_KEY_Y)
        .Const(_SC("KEY_Z"),                SQMOD_KEYCODE_KEY_Z)
        .Const(_SC("MULTIPLY"),             SQMOD_KEYCODE_MULTIPLY)
        .Const(_SC("NONAME"),               SQMOD_KEYCODE_NONAME)
        .Const(_SC("NUMPAD0"),              SQMOD_KEYCODE_NUMPAD0)
        .Const(_SC("NUMPAD1"),              SQMOD_KEYCODE_NUMPAD1)
        .Const(_SC("NUMPAD2"),              SQMOD_KEYCODE_NUMPAD2)
        .Const(_SC("NUMPAD3"),              SQMOD_KEYCODE_NUMPAD3)
        .Const(_SC("NUMPAD4"),              SQMOD_KEYCODE_NUMPAD4)
        .Const(_SC("NUMPAD5"),              SQMOD_KEYCODE_NUMPAD5)
        .Const(_SC("NUMPAD6"),              SQMOD_KEYCODE_NUMPAD6)
        .Const(_SC("NUMPAD7"),              SQMOD_KEYCODE_NUMPAD7)
        .Const(_SC("NUMPAD8"),              SQMOD_KEYCODE_NUMPAD8)
        .Const(_SC("NUMPAD9"),              SQMOD_KEYCODE_NUMPAD9)
        .Const(_SC("OEM_1"),                SQMOD_KEYCODE_OEM_1)
        .Const(_SC("OEM_102"),              SQMOD_KEYCODE_OEM_102)
        .Const(_SC("OEM_2"),                SQMOD_KEYCODE_OEM_2)
        .Const(_SC("OEM_3"),                SQMOD_KEYCODE_OEM_3)
        .Const(_SC("OEM_4"),                SQMOD_KEYCODE_OEM_4)
        .Const(_SC("OEM_5"),                SQMOD_KEYCODE_OEM_5)
        .Const(_SC("OEM_6"),                SQMOD_KEYCODE_OEM_6)
        .Const(_SC("OEM_7"),                SQMOD_KEYCODE_OEM_7)
        .Const(_SC("OEM_8"),                SQMOD_KEYCODE_OEM_8)
        .Const(_SC("OEM_ATTN"),             SQMOD_KEYCODE_OEM_ATTN)
        .Const(_SC("OEM_AUTO"),             SQMOD_KEYCODE_OEM_AUTO)
        .Const(_SC("OEM_AX"),               SQMOD_KEYCODE_OEM_AX)
        .Const(_SC("OEM_BACKTAB"),          SQMOD_KEYCODE_OEM_BACKTAB)
        .Const(_SC("OEM_CLEAR"),            SQMOD_KEYCODE_OEM_CLEAR)
        .Const(_SC("OEM_COMMA"),            SQMOD_KEYCODE_OEM_COMMA)
        .Const(_SC("OEM_COPY"),             SQMOD_KEYCODE_OEM_COPY)
        .Const(_SC("OEM_CUSEL"),            SQMOD_KEYCODE_OEM_CUSEL)
        .Const(_SC("OEM_ENLW"),             SQMOD_KEYCODE_OEM_ENLW)
        .Const(_SC("OEM_FINISH"),           SQMOD_KEYCODE_OEM_FINISH)
        .Const(_SC("OEM_FJ_LOYA"),          SQMOD_KEYCODE_OEM_FJ_LOYA)
        .Const(_SC("OEM_FJ_MASSHOU"),       SQMOD_KEYCODE_OEM_FJ_MASSHOU)
        .Const(_SC("OEM_FJ_ROYA"),          SQMOD_KEYCODE_OEM_FJ_ROYA)
        .Const(_SC("OEM_FJ_TOUROKU"),       SQMOD_KEYCODE_OEM_FJ_TOUROKU)
        .Const(_SC("OEM_JUMP"),             SQMOD_KEYCODE_OEM_JUMP)
        .Const(_SC("OEM_MINUS"),            SQMOD_KEYCODE_OEM_MINUS)
        .Const(_SC("OEM_PA1"),              SQMOD_KEYCODE_OEM_PA1)
        .Const(_SC("OEM_PA2"),              SQMOD_KEYCODE_OEM_PA2)
        .Const(_SC("OEM_PA3"),              SQMOD_KEYCODE_OEM_PA3)
        .Const(_SC("OEM_PERIOD"),           SQMOD_KEYCODE_OEM_PERIOD)
        .Const(_SC("OEM_PLUS"),             SQMOD_KEYCODE_OEM_PLUS)
        .Const(_SC("OEM_RESET"),            SQMOD_KEYCODE_OEM_RESET)
        .Const(_SC("OEM_WSCTRL"),           SQMOD_KEYCODE_OEM_WSCTRL)
        .Const(_SC("PA1"),                  SQMOD_KEYCODE_PA1)
        .Const(_SC("PACKET"),               SQMOD_KEYCODE_PACKET)
        .Const(_SC("PLAY"),                 SQMOD_KEYCODE_PLAY)
        .Const(_SC("PROCESSKEY"),           SQMOD_KEYCODE_PROCESSKEY)
        .Const(_SC("RETURN"),               SQMOD_KEYCODE_RETURN)
        .Const(_SC("SELECT"),               SQMOD_KEYCODE_SELECT)
        .Const(_SC("SEPARATOR"),            SQMOD_KEYCODE_SEPARATOR)
        .Const(_SC("SPACE"),                SQMOD_KEYCODE_SPACE)
        .Const(_SC("SUBTRACT"),             SQMOD_KEYCODE_SUBTRACT)
        .Const(_SC("TAB"),                  SQMOD_KEYCODE_TAB)
        .Const(_SC("ZOOM"),                 SQMOD_KEYCODE_ZOOM)
        .Const(_SC("ACCEPT"),               SQMOD_KEYCODE_ACCEPT)
        .Const(_SC("APPS"),                 SQMOD_KEYCODE_APPS)
        .Const(_SC("BROWSER_BACK"),         SQMOD_KEYCODE_BROWSER_BACK)
        .Const(_SC("BROWSER_FAVORITES"),    SQMOD_KEYCODE_BROWSER_FAVORITES)
        .Const(_SC("BROWSER_FORWARD"),      SQMOD_KEYCODE_BROWSER_FORWARD)
        .Const(_SC("BROWSER_HOME"),         SQMOD_KEYCODE_BROWSER_HOME)
        .Const(_SC("BROWSER_REFRESH"),      SQMOD_KEYCODE_BROWSER_REFRESH)
        .Const(_SC("BROWSER_SEARCH"),       SQMOD_KEYCODE_BROWSER_SEARCH)
        .Const(_SC("BROWSER_STOP"),         SQMOD_KEYCODE_BROWSER_STOP)
        .Const(_SC("CAPITAL"),              SQMOD_KEYCODE_CAPITAL)
        .Const(_SC("CONVERT"),              SQMOD_KEYCODE_CONVERT)
        .Const(_SC("DELETE"),               SQMOD_KEYCODE_DELETE)
        .Const(_SC("DOWN"),                 SQMOD_KEYCODE_DOWN)
        .Const(_SC("END"),                  SQMOD_KEYCODE_END)
        .Const(_SC("F1"),                   SQMOD_KEYCODE_F1)
        .Const(_SC("F10"),                  SQMOD_KEYCODE_F10)
        .Const(_SC("F11"),                  SQMOD_KEYCODE_F11)
        .Const(_SC("F12"),                  SQMOD_KEYCODE_F12)
        .Const(_SC("F13"),                  SQMOD_KEYCODE_F13)
        .Const(_SC("F14"),                  SQMOD_KEYCODE_F14)
        .Const(_SC("F15"),                  SQMOD_KEYCODE_F15)
        .Const(_SC("F16"),                  SQMOD_KEYCODE_F16)
        .Const(_SC("F17"),                  SQMOD_KEYCODE_F17)
        .Const(_SC("F18"),                  SQMOD_KEYCODE_F18)
        .Const(_SC("F19"),                  SQMOD_KEYCODE_F19)
        .Const(_SC("F2"),                   SQMOD_KEYCODE_F2)
        .Const(_SC("F20"),                  SQMOD_KEYCODE_F20)
        .Const(_SC("F21"),                  SQMOD_KEYCODE_F21)
        .Const(_SC("F22"),                  SQMOD_KEYCODE_F22)
        .Const(_SC("F23"),                  SQMOD_KEYCODE_F23)
        .Const(_SC("F24"),                  SQMOD_KEYCODE_F24)
        .Const(_SC("F3"),                   SQMOD_KEYCODE_F3)
        .Const(_SC("F4"),                   SQMOD_KEYCODE_F4)
        .Const(_SC("F5"),                   SQMOD_KEYCODE_F5)
        .Const(_SC("F6"),                   SQMOD_KEYCODE_F6)
        .Const(_SC("F7"),                   SQMOD_KEYCODE_F7)
        .Const(_SC("F8"),                   SQMOD_KEYCODE_F8)
        .Const(_SC("F9"),                   SQMOD_KEYCODE_F9)
        .Const(_SC("FINAL"),                SQMOD_KEYCODE_FINAL)
        .Const(_SC("HELP"),                 SQMOD_KEYCODE_HELP)
        .Const(_SC("HOME"),                 SQMOD_KEYCODE_HOME)
        .Const(_SC("ICO_00"),               SQMOD_KEYCODE_ICO_00)
        .Const(_SC("INSERT"),               SQMOD_KEYCODE_INSERT)
        .Const(_SC("JUNJA"),                SQMOD_KEYCODE_JUNJA)
        .Const(_SC("KANA"),                 SQMOD_KEYCODE_KANA)
        .Const(_SC("KANJI"),                SQMOD_KEYCODE_KANJI)
        .Const(_SC("LAUNCH_APP1"),          SQMOD_KEYCODE_LAUNCH_APP1)
        .Const(_SC("LAUNCH_APP2"),          SQMOD_KEYCODE_LAUNCH_APP2)
        .Const(_SC("LAUNCH_MAIL"),          SQMOD_KEYCODE_LAUNCH_MAIL)
        .Const(_SC("LAUNCH_MEDIA_SELECT"),  SQMOD_KEYCODE_LAUNCH_MEDIA_SELECT)
        .Const(_SC("LBUTTON"),              SQMOD_KEYCODE_LBUTTON)
        .Const(_SC("LCONTROL"),             SQMOD_KEYCODE_LCONTROL)
        .Const(_SC("LEFT"),                 SQMOD_KEYCODE_LEFT)
        .Const(_SC("LMENU"),                SQMOD_KEYCODE_LMENU)
        .Const(_SC("LSHIFT"),               SQMOD_KEYCODE_LSHIFT)
        .Const(_SC("LWIN"),                 SQMOD_KEYCODE_LWIN)
        .Const(_SC("MBUTTON"),              SQMOD_KEYCODE_MBUTTON)
        .Const(_SC("MEDIA_NEXT_TRACK"),     SQMOD_KEYCODE_MEDIA_NEXT_TRACK)
        .Const(_SC("MEDIA_PLAY_PAUSE"),     SQMOD_KEYCODE_MEDIA_PLAY_PAUSE)
        .Const(_SC("MEDIA_PREV_TRACK"),     SQMOD_KEYCODE_MEDIA_PREV_TRACK)
        .Const(_SC("MEDIA_STOP"),           SQMOD_KEYCODE_MEDIA_STOP)
        .Const(_SC("MODECHANGE"),           SQMOD_KEYCODE_MODECHANGE)
        .Const(_SC("NEXT"),                 SQMOD_KEYCODE_NEXT)
        .Const(_SC("NONCONVERT"),           SQMOD_KEYCODE_NONCONVERT)
        .Const(_SC("NUMLOCK"),              SQMOD_KEYCODE_NUMLOCK)
        .Const(_SC("OEM_FJ_JISHO"),         SQMOD_KEYCODE_OEM_FJ_JISHO)
        .Const(_SC("PAUSE"),                SQMOD_KEYCODE_PAUSE)
        .Const(_SC("PRINT"),                SQMOD_KEYCODE_PRINT)
        .Const(_SC("PRIOR"),                SQMOD_KEYCODE_PRIOR)
        .Const(_SC("RBUTTON"),              SQMOD_KEYCODE_RBUTTON)
        .Const(_SC("RCONTROL"),             SQMOD_KEYCODE_RCONTROL)
        .Const(_SC("RIGHT"),                SQMOD_KEYCODE_RIGHT)
        .Const(_SC("RMENU"),                SQMOD_KEYCODE_RMENU)
        .Const(_SC("RSHIFT"),               SQMOD_KEYCODE_RSHIFT)
        .Const(_SC("RWIN"),                 SQMOD_KEYCODE_RWIN)
        .Const(_SC("SCROLL"),               SQMOD_KEYCODE_SCROLL)
        .Const(_SC("SLEEP"),                SQMOD_KEYCODE_SLEEP)
        .Const(_SC("SNAPSHOT"),             SQMOD_KEYCODE_SNAPSHOT)
        .Const(_SC("UP"),                   SQMOD_KEYCODE_UP)
        .Const(_SC("VOLUME_DOWN"),          SQMOD_KEYCODE_VOLUME_DOWN)
        .Const(_SC("VOLUME_MUTE"),          SQMOD_KEYCODE_VOLUME_MUTE)
        .Const(_SC("VOLUME_UP"),            SQMOD_KEYCODE_VOLUME_UP)
        .Const(_SC("XBUTTON1"),             SQMOD_KEYCODE_XBUTTON1)
        .Const(_SC("XBUTTON2"),             SQMOD_KEYCODE_XBUTTON2)
        .Const(_SC("NONE"),                 SQMOD_KEYCODE_NONE)
    );

    Sqrat::ConstTable(vm).Enum(_SC("EASCII"), Sqrat::Enumeration(vm)
        .Const(_SC("NUL"),                  SQMOD_ASCII_NUL)
        .Const(_SC("SOH"),                  SQMOD_ASCII_SOH)
        .Const(_SC("STX"),                  SQMOD_ASCII_STX)
        .Const(_SC("ETX"),                  SQMOD_ASCII_ETX)
        .Const(_SC("EOT"),                  SQMOD_ASCII_EOT)
        .Const(_SC("ENQ"),                  SQMOD_ASCII_ENQ)
        .Const(_SC("ACK"),                  SQMOD_ASCII_ACK)
        .Const(_SC("BEL"),                  SQMOD_ASCII_BEL)
        .Const(_SC("BS"),                   SQMOD_ASCII_BS)
        .Const(_SC("TAB"),                  SQMOD_ASCII_TAB)
        .Const(_SC("LF"),                   SQMOD_ASCII_LF)
        .Const(_SC("VT"),                   SQMOD_ASCII_VT)
        .Const(_SC("FF"),                   SQMOD_ASCII_FF)
        .Const(_SC("CR"),                   SQMOD_ASCII_CR)
        .Const(_SC("SO"),                   SQMOD_ASCII_SO)
        .Const(_SC("SI"),                   SQMOD_ASCII_SI)
        .Const(_SC("DLE"),                  SQMOD_ASCII_DLE)
        .Const(_SC("DC1"),                  SQMOD_ASCII_DC1)
        .Const(_SC("DC2"),                  SQMOD_ASCII_DC2)
        .Const(_SC("DC3"),                  SQMOD_ASCII_DC3)
        .Const(_SC("DC4"),                  SQMOD_ASCII_DC4)
        .Const(_SC("NAK"),                  SQMOD_ASCII_NAK)
        .Const(_SC("SYN"),                  SQMOD_ASCII_SYN)
        .Const(_SC("ETB"),                  SQMOD_ASCII_ETB)
        .Const(_SC("CAN"),                  SQMOD_ASCII_CAN)
        .Const(_SC("EM"),                   SQMOD_ASCII_EM)
        .Const(_SC("SUB"),                  SQMOD_ASCII_SUB)
        .Const(_SC("ESC"),                  SQMOD_ASCII_ESC)
        .Const(_SC("FS"),                   SQMOD_ASCII_FS)
        .Const(_SC("GS"),                   SQMOD_ASCII_GS)
        .Const(_SC("RS"),                   SQMOD_ASCII_RS)
        .Const(_SC("US"),                   SQMOD_ASCII_US)
        .Const(_SC("SPACE"),                SQMOD_ASCII_SPACE)
        .Const(_SC("EXCLAMATION_POINT"),    SQMOD_ASCII_EXCLAMATION_POINT)
        .Const(_SC("DOUBLE_QUOTES"),        SQMOD_ASCII_DOUBLE_QUOTES)
        .Const(_SC("NUMBER_SIGN"),          SQMOD_ASCII_NUMBER_SIGN)
        .Const(_SC("DOLLAR_SIGN"),          SQMOD_ASCII_DOLLAR_SIGN)
        .Const(_SC("PERCENT_SIGN"),         SQMOD_ASCII_PERCENT_SIGN)
        .Const(_SC("AMPERSAND"),            SQMOD_ASCII_AMPERSAND)
        .Const(_SC("SINGLE_QUOTE"),         SQMOD_ASCII_SINGLE_QUOTE)
        .Const(_SC("OPENING_PARENTHESIS"),  SQMOD_ASCII_OPENING_PARENTHESIS)
        .Const(_SC("CLOSING_PARENTHESIS"),  SQMOD_ASCII_CLOSING_PARENTHESIS)
        .Const(_SC("ASTERISK"),             SQMOD_ASCII_ASTERISK)
        .Const(_SC("PLUS"),                 SQMOD_ASCII_PLUS)
        .Const(_SC("COMMA"),                SQMOD_ASCII_COMMA)
        .Const(_SC("MINUS"),                SQMOD_ASCII_MINUS)
        .Const(_SC("PERIOD"),               SQMOD_ASCII_PERIOD)
        .Const(_SC("SLASH"),                SQMOD_ASCII_SLASH)
        .Const(_SC("ZERO"),                 SQMOD_ASCII_ZERO)
        .Const(_SC("ONE"),                  SQMOD_ASCII_ONE)
        .Const(_SC("TWO"),                  SQMOD_ASCII_TWO)
        .Const(_SC("THREE"),                SQMOD_ASCII_THREE)
        .Const(_SC("FOUR"),                 SQMOD_ASCII_FOUR)
        .Const(_SC("FIVE"),                 SQMOD_ASCII_FIVE)
        .Const(_SC("SIX"),                  SQMOD_ASCII_SIX)
        .Const(_SC("SEVEN"),                SQMOD_ASCII_SEVEN)
        .Const(_SC("EIGHT"),                SQMOD_ASCII_EIGHT)
        .Const(_SC("NINE"),                 SQMOD_ASCII_NINE)
        .Const(_SC("COLON"),                SQMOD_ASCII_COLON)
        .Const(_SC("EMICOLON"),             SQMOD_ASCII_EMICOLON)
        .Const(_SC("LESS_THAN_SIGN"),       SQMOD_ASCII_LESS_THAN_SIGN)
        .Const(_SC("EQUAL_SIGN"),           SQMOD_ASCII_EQUAL_SIGN)
        .Const(_SC("GREATER_THAN_SIGN"),    SQMOD_ASCII_GREATER_THAN_SIGN)
        .Const(_SC("QUESTION_MARK"),        SQMOD_ASCII_QUESTION_MARK)
        .Const(_SC("AT"),                   SQMOD_ASCII_AT)
        .Const(_SC("UPPER_A"),              SQMOD_ASCII_UPPER_A)
        .Const(_SC("UPPER_B"),              SQMOD_ASCII_UPPER_B)
        .Const(_SC("UPPER_C"),              SQMOD_ASCII_UPPER_C)
        .Const(_SC("UPPER_D"),              SQMOD_ASCII_UPPER_D)
        .Const(_SC("UPPER_E"),              SQMOD_ASCII_UPPER_E)
        .Const(_SC("UPPER_F"),              SQMOD_ASCII_UPPER_F)
        .Const(_SC("UPPER_G"),              SQMOD_ASCII_UPPER_G)
        .Const(_SC("UPPER_H"),              SQMOD_ASCII_UPPER_H)
        .Const(_SC("UPPER_I"),              SQMOD_ASCII_UPPER_I)
        .Const(_SC("UPPER_J"),              SQMOD_ASCII_UPPER_J)
        .Const(_SC("UPPER_K"),              SQMOD_ASCII_UPPER_K)
        .Const(_SC("UPPER_L"),              SQMOD_ASCII_UPPER_L)
        .Const(_SC("UPPER_M"),              SQMOD_ASCII_UPPER_M)
        .Const(_SC("UPPER_N"),              SQMOD_ASCII_UPPER_N)
        .Const(_SC("UPPER_O"),              SQMOD_ASCII_UPPER_O)
        .Const(_SC("UPPER_P"),              SQMOD_ASCII_UPPER_P)
        .Const(_SC("UPPER_Q"),              SQMOD_ASCII_UPPER_Q)
        .Const(_SC("UPPER_R"),              SQMOD_ASCII_UPPER_R)
        .Const(_SC("UPPER_S"),              SQMOD_ASCII_UPPER_S)
        .Const(_SC("UPPER_T"),              SQMOD_ASCII_UPPER_T)
        .Const(_SC("UPPER_U"),              SQMOD_ASCII_UPPER_U)
        .Const(_SC("UPPER_V"),              SQMOD_ASCII_UPPER_V)
        .Const(_SC("UPPER_W"),              SQMOD_ASCII_UPPER_W)
        .Const(_SC("UPPER_X"),              SQMOD_ASCII_UPPER_X)
        .Const(_SC("UPPER_Y"),              SQMOD_ASCII_UPPER_Y)
        .Const(_SC("UPPER_Z"),              SQMOD_ASCII_UPPER_Z)
        .Const(_SC("OPENING_BRACKET"),      SQMOD_ASCII_OPENING_BRACKET)
        .Const(_SC("BACKSLASH"),            SQMOD_ASCII_BACKSLASH)
        .Const(_SC("CLOSING_BRACKET"),      SQMOD_ASCII_CLOSING_BRACKET)
        .Const(_SC("CARET"),                SQMOD_ASCII_CARET)
        .Const(_SC("UNDERSCORE"),           SQMOD_ASCII_UNDERSCORE)
        .Const(_SC("GRAVE_ACCENT"),         SQMOD_ASCII_GRAVE_ACCENT)
        .Const(_SC("LOWER_A"),              SQMOD_ASCII_LOWER_A)
        .Const(_SC("LOWER_B"),              SQMOD_ASCII_LOWER_B)
        .Const(_SC("LOWER_C"),              SQMOD_ASCII_LOWER_C)
        .Const(_SC("LOWER_D"),              SQMOD_ASCII_LOWER_D)
        .Const(_SC("LOWER_E"),              SQMOD_ASCII_LOWER_E)
        .Const(_SC("LOWER_F"),              SQMOD_ASCII_LOWER_F)
        .Const(_SC("LOWER_G"),              SQMOD_ASCII_LOWER_G)
        .Const(_SC("LOWER_H"),              SQMOD_ASCII_LOWER_H)
        .Const(_SC("LOWER_I"),              SQMOD_ASCII_LOWER_I)
        .Const(_SC("LOWER_J"),              SQMOD_ASCII_LOWER_J)
        .Const(_SC("LOWER_K"),              SQMOD_ASCII_LOWER_K)
        .Const(_SC("LOWER_L"),              SQMOD_ASCII_LOWER_L)
        .Const(_SC("LOWER_M"),              SQMOD_ASCII_LOWER_M)
        .Const(_SC("LOWER_N"),              SQMOD_ASCII_LOWER_N)
        .Const(_SC("LOWER_O"),              SQMOD_ASCII_LOWER_O)
        .Const(_SC("LOWER_P"),              SQMOD_ASCII_LOWER_P)
        .Const(_SC("LOWER_Q"),              SQMOD_ASCII_LOWER_Q)
        .Const(_SC("LOWER_R"),              SQMOD_ASCII_LOWER_R)
        .Const(_SC("LOWER_S"),              SQMOD_ASCII_LOWER_S)
        .Const(_SC("LOWER_T"),              SQMOD_ASCII_LOWER_T)
        .Const(_SC("LOWER_U"),              SQMOD_ASCII_LOWER_U)
        .Const(_SC("LOWER_V"),              SQMOD_ASCII_LOWER_V)
        .Const(_SC("LOWER_W"),              SQMOD_ASCII_LOWER_W)
        .Const(_SC("LOWER_X"),              SQMOD_ASCII_LOWER_X)
        .Const(_SC("LOWER_Y"),              SQMOD_ASCII_LOWER_Y)
        .Const(_SC("LOWER_Z"),              SQMOD_ASCII_LOWER_Z)
        .Const(_SC("OPENING_BRACE"),        SQMOD_ASCII_OPENING_BRACE)
        .Const(_SC("VERTICAL_BAR"),         SQMOD_ASCII_VERTICAL_BAR)
        .Const(_SC("CLOSING_BRACE"),        SQMOD_ASCII_CLOSING_BRACE)
        .Const(_SC("TILDE"),                SQMOD_ASCII_TILDE)
        .Const(_SC("UNDEFINED"),            SQMOD_ASCII_UNDEFINED)
    );

    return true;
}

} // Namespace:: SqMod
